part of tecfy_corev2_client_package;

class EntityListTileComponent extends StatelessWidget {
  final Entity entity;
  final Map<String, dynamic> data;
  Property? titleField;
  Property? subTitleField;
  Property? tailField;
  final void Function(Map<String, dynamic>)? onExpandItem;
  final void Function(Map<String, dynamic>?)? onClick;
  final void Function(Map<String, dynamic>?)? onLongClick;
  final bool showUpdateButton;
  EntityListTileComponent(this.entity, this.data,
      {this.onClick,
      required this.showUpdateButton,
      this.onExpandItem,
      this.onLongClick}) {
    // TODO: move these properties to [Entity] class to save calling them everytime need to print list item
    var props =
        entity.gridProperties; //.where((e) => e.dbName != 'active').toList();
    var dis = entity.gridProperties.where((e) => e.isDisplayField).toList();
    props.sort((a, b) => a.gridOrder - b.gridOrder);
    if (dis.length > 0) {
      dis.sort((a, b) => a.gridOrder - b.gridOrder);
      titleField = dis[0];
      props = props.where((e) => e.dbName != titleField?.dbName).toList();
      if (dis.length > 1)
        subTitleField = dis[1];
      else
        subTitleField = props.length > 0 ? props[0] : null;
      if (dis.length > 2)
        subTitleField = dis[2];
      else
        tailField = props.length > 1 ? props[1] : null;
      if (tailField == null && subTitleField != null) {
        tailField = subTitleField;
        subTitleField = null;
      }
    } else {
      titleField = props.length > 0 ? props[0] : null;
      subTitleField = props.length > 1 ? props[1] : null;
      tailField = props.length > 2 ? props[2] : null;
      if (tailField == null && subTitleField != null) {
        tailField = subTitleField;
        subTitleField = null;
      }
    }
  }

  Widget _getTreeCell() {
    if ((data['_children'] != null && data['_children'].length == 0) ||
        onExpandItem == null)
      return SizedBox(
          width: 30 + (((data['_tree_level'] ?? 0) as num).toDouble() * 10.0));
    return IconButton(
      tooltip: data['_expanded'] == true ? 'Collapse' : 'Expand',
      padding: EdgeInsets.only(
          right: LocalConfigService.isArabic
              ? ((data['_tree_level'] ?? 0) as num).toDouble() * 10.0
              : 0.0,
          left: LocalConfigService.isArabic
              ? 0.0
              : ((data['_tree_level'] ?? 0) as num).toDouble() * 10.0),
      icon: data['_expanded'] == true
          ? Icon(Icons.arrow_drop_down, size: 26)
          : RotatedBox(
              quarterTurns: 2,
              child: Icon(Icons.play_arrow_outlined, size: 20),
            ),
      onPressed: () => onExpandItem?.call(data),
    );
  }

  _onSaveNewItem(BuildContext context, newDataRaw) async {
    try {
      var validateResult =
          EntityService.validateEntity(entity, newDataRaw, true);
      if (!validateResult.success) {
        AppNavigator.showMessage(
            context, validateResult.message ?? 'Error', MessageType.error);
        return;
      }
      var addResult =
          await EntityService.saveEntity(entity, newDataRaw, context, false);
      if (addResult.success) {
        Navigator.pop(context, true); // close the add new item's form
      } else {
        AppNavigator.showMessage(
            context, addResult.message ?? '', MessageType.error);
      }
    } catch (err, trace) {
      print('XXXXXXXXX erorr adding item');
      print(err);
      print(trace);
      AppNavigator.showMessage(context, 'Error Adding Item', MessageType.error);
    }
  }

  Widget _getAddNewDialog(addDialogContext, newDataRaw) {
    return Container(
      constraints: BoxConstraints(maxHeight: 500, maxWidth: 500),
      child: Center(
        child: Card(
          child: ListView(
            children: <Widget>[
              InLineForm(entity.dbName, true,
                  entity: entity,
                  inPopup: true,
                  data: newDataRaw,
                  onChanged: (v) {}),
              Padding(
                  padding: EdgeInsets.symmetric(horizontal: 20),
                  child: Row(
                    children: <Widget>[
                      Expanded(child: Container()),
                      Expanded(
                          child: AppButtonPost(
                              label: 'Save',
                              onPressed: () => _onSaveNewItem(
                                  addDialogContext, newDataRaw))),
                      SizedBox(width: 10),
                      Expanded(
                          child: Container(
                              height: 50.0,
                              child: TextButton(
                                  child: TecfyText('Cancel',
                                      style: ThemeService.flatButtonTextStyle),
                                  onPressed: () =>
                                      Navigator.pop(addDialogContext, false)))),
                      Expanded(child: Container()),
                    ],
                  )),
            ],
          ),
        ),
      ),
    );
  }

  Widget _editButton(context) {
    return IconButton(
        onPressed: () async {
          // Map<String, dynamic> newDataRaw =
          //     new Map<String, dynamic>();
          var addNewResult = await AppNavigator.showAppDialogEx(
              context, () => _getAddNewDialog(context, data),
              size: DialogBoxSizes.large);
          if (addNewResult ?? false)
            Navigator.pop(context); // close the search form
        },
        tooltip: 'Edit',
        icon: Icon(Icons.note));
  }

  @override
  Widget build(BuildContext context) {
    return ListTile(
        leading: entity.isTree && onExpandItem != null
            ? _getTreeCell()
            : entity.isTree
                ? Container(
                    width: 115,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        IconButton(
                            onPressed: () => onClick?.call(data),
                            tooltip: 'Subcategory',
                            icon: Icon(Icons.subdirectory_arrow_left)),
                        entity.hasPhoto
                            ? PhotoViewProperty(
                                data: data[entity.photoPropName].toString())
                            : Container(),
                      ],
                    ))
                : entity.hasPhoto
                    ? PhotoViewProperty(
                        data: data[entity.photoPropName].toString())
                    : null,
        title: titleField == null
            ? null
            : Padding(
                padding: EdgeInsets.only(
                    right: LocalConfigService.isArabic
                        ? ((data['_tree_level'] ?? 0) as num).toDouble() * 10.0
                        : 0.0,
                    left: LocalConfigService.isArabic
                        ? 0.0
                        : ((data['_tree_level'] ?? 0) as num).toDouble() *
                            10.0),
                child: titleField!.getDisplayWidget(data,
                    textStyle: TextStyle(
                        fontSize: 16,
                        color: Colors
                            .black))), //, style:TextStyle(color: data['active'] == false ? Colors.red : Colors.black)),
        subtitle: subTitleField == null
            ? null
            : Padding(
                padding: EdgeInsets.only(
                    right: LocalConfigService.isArabic
                        ? ((data['_tree_level'] ?? 0) as num).toDouble() * 10
                        : 0,
                    left: LocalConfigService.isArabic
                        ? 0
                        : ((data['_tree_level'] ?? 0) as num).toDouble() * 10),
                child: subTitleField!.getDisplayWidget(data,
                    textStyle: TextStyle(fontSize: 14, color: Colors.black54))),
        trailing:
            showUpdateButton ? _editButton(context) : tailField?.getDisplayWidget(data, textStyle: TextStyle(fontSize: 10, color: Colors.black87)),
        onTap: () => onClick?.call(data),
        onLongPress: () => onLongClick?.call(data));
  }
}
