part of tecfy_corev2_client_package;

class ReportDetailPage extends StatefulWidget {
  /// the report name
  final Map<String, dynamic> report;

  /// the filter properties
  final List<EntityFilter> filter;

  /// if true only body shown without BasePage
  final bool isLite;

  /// view report page with filter and print
  ReportDetailPage(this.report, this.filter, {this.isLite = false});
  @override
  _ReportDetailPageState createState() => _ReportDetailPageState();
}

class _ReportDetailPageState extends TecfyState<ReportDetailPage> {
  List<Map<String, dynamic>>? _data;
  Map<String, List<Map<String, dynamic>>>? _dataGrouped;
  Map<String, Map<String, dynamic>>? _dataGroupedTotal;
  List<Property>? _filters;
  Map<String, dynamic>? _filterData;
  Map<String, dynamic>? _totals;
  late List<EntityFilter> reportFilters;
  int? _sortColumnIndex;
  bool _sortAcd = true;
  ByteData? _font;
  late pw.TextStyle textStyleBoldPdf;
  late pw.TextStyle textStylePdf;
  late List<ReportProperty> _props;
  late List<ReportProperty>? _groupProperties;
  String? _groupBy;
  bool showGrouped = false;

  // late List<ReportProperty> _propsSortedForPdf;
  late ScrollController _scrollControllerH;
  late ScrollController _scrollControllerV;
  String _mode = "portrait";
  bool isEditing = false;
  late Map savedData;
  List<ReportProperty> savedList = [];

  @override
  void initState() {
    reportFilters = widget.filter;
    _scrollControllerV = ScrollController();
    _scrollControllerH = ScrollController();
    rootBundle
        .load("packages/tecfy_corev2_package/assets/fonts/STC-Regular.ttf")
        .then((d) {
      _font = d;
      textStyleBoldPdf = pw.TextStyle(
          font: pw.Font.ttf(_font!),
          fontSize: 13,
          fontWeight: pw.FontWeight.bold);
      textStylePdf = pw.TextStyle(font: pw.Font.ttf(_font!));
    });
    widget.report['properties'] = widget.report['properties']
        .map((e) => Map<String, dynamic>.from(e))
        .toList();
    _props = widget.report['properties']
        .map<ReportProperty>((p) => ReportProperty.fromMap(p))
        .toList();
    _groupBy = widget.report['groupBy'];
    _groupProperties = widget.report['groupProperties']
        ?.map((r) => _props
            .map((t) => t as ReportProperty?)
            .firstWhere((e) => e?.dbName == r, orElse: () => null))
        .where((r) => r != null)
        .map<ReportProperty>((t) => t as ReportProperty)
        .toList();
    // _props = _props
    //     .where((element) =>
    //         _groupProperties?.any((p) => p.dbName == element.dbName) != true)
    //     .toList();
    savedData = ConfigService.getValueMap('Report_${widget.report['dbName']}');
    print("SavedList=========>${savedData}");
    if (savedData.isNotEmpty) {
      savedData.entries.forEach((element) {
        savedList.add(ReportProperty(
            dbName: element.value['dbName'],
            nameEn: element.value['nameEn'],
            nameAr: element.value['nameAr'],
            type: ValueTypes.values[element.value['type'] ?? 0]));
      });
      savedList.remove(savedList[savedList.length - 1]);
      savedList.remove(savedList.last);
      print("SavedList=========>${savedList}");
    } else {
      print("_props=========>${_props.map}");

      savedList = _props.toList();
      print("SavedList=========>${savedList}");
    }

    _getData();

    super.initState();
  }

  @override
  void dispose() {
    _scrollControllerH.dispose();
    _scrollControllerV.dispose();
    super.dispose();
  }

  void _onSort(String dbName, ValueTypes type) {
    if (_dataGrouped != null && _groupProperties != null) {
      for (var d in _dataGrouped!.values) {
        d.sort((a, b) => _sortAcd
            ? (type == ValueTypes.Number
                ? (a[dbName] - b[dbName]).toInt()
                : a[dbName].toString().compareTo(b[dbName].toString()))
            : (type == ValueTypes.Number
                ? (b[dbName] - a[dbName]).toInt()
                : b[dbName].toString().compareTo(a[dbName].toString())));
      }
    } else {
      _data?.sort((a, b) => _sortAcd
          ? (type == ValueTypes.Number
              ? (a[dbName] - b[dbName]).toInt()
              : a[dbName].toString().compareTo(b[dbName].toString()))
          : (type == ValueTypes.Number
              ? (b[dbName] - a[dbName]).toInt()
              : b[dbName].toString().compareTo(a[dbName].toString())));
    }
    setState(() {});
  }

  Future _filter() async {
    List<EntityFilter> filter = [];
    bool canceled = false;
    if (widget.report['filters'].length > 0) {
      var result = await AppNavigator.showAppDialog(
          context,
          (c) => EntityFilterPage(
              widget.report['filters'].map<Property>((r) {
                var prop = Property.getProperty(r);
                //prop.isVisable = true;
                prop?.isAdd = true;
                prop?.isEdit = true;
                //prop.size = true;
                prop?.isDisplayField = true;
                prop?.isGrid = false;
                prop?.readOnly = false;
                prop?.isRequired = r['isRequired'] ?? false;
                return prop!;
              }).toList(),
              reportFilters));
      if (result == null) canceled = true;
      if ((result as List<EntityFilter>?) == null)
        result = List.empty(growable: true);
      filter = result
              ?.where((f) =>
                  f.value != null &&
                  f.property != null &&
                  f.propertyName != null &&
                  f.operatorId != null)
              .toList() ??
          [];
    }

    if (canceled) return;

    reportFilters = filter;
    await _getData();
  }

  Future _getData() async {
    try {
      if (_data != null) {
        _data = null;
        _dataGrouped = null;
        _dataGroupedTotal = null;
        _totals = null;
        setState(() {});
      }
      //List<dynamic> data;
      // if (ConnectionService.isOfflineMode) {
      //   var result = await EntityService().getData(
      //       EntityService.getEntity(widget.report['entityName']),
      //       filter: reportFilters,
      //       allRecords: true);
      //   data = result.data;
      // } else {
      if (TecfyCoreApp.reportService != null &&
          TecfySyncService.isOfflineMode &&
          widget.report['isOnlineOnly'] == false) {
        _data = (await TecfyCoreApp.reportService
                ?.getReportData(widget.report['dbName'], reportFilters))
            as List<Map<String, dynamic>>?;
      } else {
        var result = await TecfyBasicApp.apiService.httpGetDynamic(getUrlEx(
                '/api/report/setup/getReport/${widget.report['_id']}?')! +
            (reportFilters.length > 0
                ? '&filter=${jsonEncode(reportFilters.map<Map>((f) => f.toJson()).toList(), toEncodable: EntityService.customEncode)}'
                : ''));
        if (!result.success) {
          await result.showMessage(context);
          return;
        }
        if (result.data != null) {
          if (result.data.isNotEmpty) {
            _data = [];
            for (var element in result.data) {
              if (element != null) {
                _data?.add(element as Map<String, dynamic>);
              } else {
                continue;
              }
            }
          }
        }
        // _data = result.data['data']
        //     .map<Map<String, dynamic>>((e) => e as Map<String, dynamic>)
        //     .toList();
        // _data?.removeLast();
        // if (result.data['filter'] != null) {
        //   if (result.data['filter'].isNotEmpty) {
        //     for (var element in result.data['filter']) {
        //       if (element != null) {
        //         _filters?.add(Property.getProperty(element)!);
        //       } else {
        //         continue;
        //       }
        //     }
        //   }
        // }
        // _filters = result.data['filter']
        //     .map<Property>((e) => Property.getProperty(e)!)
        //     .toList();
        // if (result.data?['filterData'] != null) {
        //   _filterData = result.data?['filterData'] as Map<String, dynamic>?;
        // }
        // _filterData = result.data?['filterData'] as Map<String, dynamic>?;
      }
      // }
      if (_data != null) {
        // if (widget.report['totals'] != null &&
        //     widget.report['totals'].length > 0) {
        //   totals = {};
        //   for (var t in widget.report['totals']) {
        //     totals[t] = 0;
        //     for (var d in data) {
        //       totals[t] += d[t] ?? 0;
        //     }
        //   }
        // }

        if (_groupBy != null) {
          print('groupBy: $_groupBy');
          _dataGrouped ??= {};
          for (var d in _data ?? []) {
            print(d[_groupBy]);
            _dataGrouped?[d[_groupBy]] ??= [];
            _dataGrouped?[d[_groupBy]]?.add(d);
          }
          _dataGroupedTotal ??= {};
          for (var d in _dataGrouped!.values) {
            var last = d.removeLast();
            _dataGroupedTotal![last[_groupBy]] = last;
          }
        }
        _totals = {}; //_data?.removeLast();
        setState(() {});
      } else {
        await AppNavigator.showMessage(
            context, "Error or No Data!", MessageType.error);
        Navigator.pop(context);
      }

      print("SavedListXXXX=========>${savedData}");
    } catch (err, trace) {
      print('XXXXXXXXXXXXXXXXXXX report load error');
      print(err);
      print(trace);
      await AppNavigator.showMessage(
          context, "Error getting data!", MessageType.error);
      Navigator.pop(context);
    }
  }

  List<DataRow> _getRows(data, [Map<String, dynamic>? totals]) {
    var result = data!
        .map<DataRow>((d) => DataRow(
              cells: savedList
                  .where((element) =>
                      !showGrouped ||
                      _groupProperties
                              ?.any((p) => p.dbName == element.dbName) !=
                          true)
                  .map<DataCell>((p) => _getCell(p, d))
                  .toList(),
            ))
        .toList();

    result.add(DataRow(
      cells: savedList
          .where((element) =>
              !showGrouped ||
              _groupProperties?.any((p) => p.dbName == element.dbName) != true)
          .map<DataCell>((p) => _getCell(p,
              (!showGrouped && _dataGrouped != null ? null : totals ?? _totals),
              totals: true))
          .toList(),
    ));
    return result;
  }

  @override
  Widget build(BuildContext context) {
    if (widget.isLite) {
      return Scaffold(
          body: Column(
        children: [
          Row(
            children: appBarActionsWidget,
          ),
          Expanded(child: childWidget)
        ],
      ));
    } else {
      return BasePage(
          context,
          (MapService.getValue(widget.report, 'name') as Map<String, dynamic>)[
              LocalConfigService.languagePrefix.toLowerCase()],
          'report',
          isMobile: LayoutService.isMobile(context),
          appBarActions: appBarActionsWidget,
          child: childWidget);
    }
  }

  List<Widget> get appBarActionsWidget => [
        if (_dataGrouped != null)
          IconButton(
              onPressed: () => setState(() => showGrouped = !showGrouped),
              icon: Icon(Icons.sticky_note_2_outlined)),
        IconButton(onPressed: _getData, icon: Icon(Icons.refresh)),
        IconButton(onPressed: _filter, icon: Icon(Icons.filter_alt_outlined)),
        AppButtonIconText(
          label: trans('Print'),
          icon: Icon(Icons.print, color: Colors.white),
          onPressed: _onPrint,
        ),
        SizedBox(width: 5),
        AppButtonIconText(
          label: 'PDF',
          icon: Icon(Icons.picture_as_pdf, color: Colors.white),
          onPressed: _onSharePDF,
        ),
        PopupMenuButton<String>(
          onSelected: (val) async {
            setState(() => _mode = val);
            if (val == 'Show All Properties') {
              await AppNavigator.showAppDialogEx(
                  context,
                  () => StatefulBuilder(
                        builder: (context, setStateDialog) => Column(
                          children: [
                            Column(
                              children: _props
                                  .map((e) => Padding(
                                      padding: EdgeInsets.all(10),
                                      child: Row(
                                          mainAxisAlignment:
                                              MainAxisAlignment.spaceAround,
                                          children: [
                                            Text(e.name),
                                            Spacer(),
                                            Checkbox(
                                                value: savedList.any((p) =>
                                                    p.dbName == e.dbName),
                                                onChanged: (val) async {
                                                  if (val == true) {
                                                    print('Adding');
                                                    savedList.add(e);
                                                    savedList.sort((a, b) =>
                                                        _props
                                                            .indexOf(a)
                                                            .compareTo(_props
                                                                .indexOf(b)));
                                                  } else {
                                                    print('Removing Column');
                                                    savedList.remove(e);
                                                  }
                                                  setState(() {});
                                                  setStateDialog(() {});
                                                }),
                                          ])))
                                  .toList(),
                            ),
                            Spacer(),
                            Padding(
                              padding: EdgeInsets.all(20),
                              child: Center(
                                child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceEvenly,
                                  crossAxisAlignment: CrossAxisAlignment.center,
                                  children: [
                                    AppButtonIconText(
                                      onPressed: () async {
                                        bool confirm =
                                            await AppNavigator.confirmMessage(
                                                context,
                                                'هل تريد '
                                                'حفظ '
                                                'الجدول بعد التعديل ؟');
                                        if (confirm) {
                                          ReportDataService.saveTableColumnSort(
                                              isEditing,
                                              savedList,
                                              context,
                                              widget.report,
                                              _sortColumnIndex ?? 0,
                                              _sortAcd);
                                        }
                                        setState(() {});
                                      },
                                      label: trans('Save Sort'),
                                    ),
                                    AppButtonIconText(
                                      onPressed: () async {
                                        setState(() {
                                          AppNavigator.pop(context);
                                        });
                                      },
                                      label: trans('Cancel'),
                                    ),
                                  ],
                                ),
                              ),
                            ),
                          ],
                        ),
                      ));
            }
          },
          itemBuilder: (BuildContext context) => [
            PopupMenuItem<String>(
              value: 'portrait',
              child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Checkbox(
                      value: _mode == 'portrait' ? true : false,
                      onChanged: (value) {},
                      activeColor: Colors.black54,
                    ),
                    Text(LocalConfigService.isArabic ? "رأسي" : "portrait",
                        style: TextStyle(
                            fontSize: 15, fontWeight: FontWeight.bold)),
                  ]),
            ),
            PopupMenuItem<String>(
              value: 'landscape',
              child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Checkbox(
                      value: _mode == 'landscape' ? true : false,
                      onChanged: (value) {},
                      activeColor: Colors.black54,
                    ),
                    Text(LocalConfigService.isArabic ? 'أفقي' : "Landscape",
                        style: TextStyle(
                            fontSize: 15, fontWeight: FontWeight.bold)),
                  ]),
            ),
            PopupMenuItem<String>(
              value: 'Edit Columns',
              onTap: () async {
                isEditing = !isEditing;
                print(isEditing);
                if (!isEditing) {
                  savedData = await ReportDataService.saveTableColumnSort(
                      isEditing,
                      savedList,
                      context,
                      widget.report,
                      _sortColumnIndex ?? 0,
                      _sortAcd);
                  // savedList=[];
                  // if(isEditing){
                  //   savedData.entries.forEach((element) {
                  //     print(element.value['type']);
                  //     savedList.add(ReportProperty
                  //       (dbName: element.value['dbName'],nameEn: element.value['nameEn'],
                  //         nameAr:
                  //         element.value['nameAr'],type: ValueTypes.values[element
                  //             .value['type']??0]));
                  //   });
                  //   savedList.remove(savedList.last);
                  //   savedList.forEach((element) {
                  //     print('${element.dbName} -- ${element.nameEn} -- ${element.type}');
                  //   });
                  // }

                  setState(() {});
                }
              },
              child: Row(mainAxisAlignment: MainAxisAlignment.end, children: [
                Text(
                    isEditing
                        ? LocalConfigService.isArabic
                            ? 'حفظ الترتيب'
                            : "Save "
                                "Sort"
                        : LocalConfigService.isArabic
                            ? "تعديل/ حذف أعمده"
                            : "Edit/Delete Columns",
                    style:
                        TextStyle(fontSize: 15, fontWeight: FontWeight.bold)),
              ]),
            ),
            PopupMenuItem<String>(
              value: 'Show All Properties',
              child: Row(mainAxisAlignment: MainAxisAlignment.end, children: [
                Text(
                    LocalConfigService.isArabic
                        ? "عرض جميع الأعمده"
                        : "Sh"
                            "ow All "
                            "Properties",
                    style:
                        TextStyle(fontSize: 15, fontWeight: FontWeight.bold)),
              ]),
            ),
            PopupMenuItem<String>(
              value: 'Clear all Settings',
              onTap: () async {
                savedList = _props;
                await ConfigService.setValueMap(
                    'Report_${widget.report['dbName']}', {});
              },
              child: Row(mainAxisAlignment: MainAxisAlignment.end, children: [
                Text(
                    LocalConfigService.isArabic
                        ? "الرجوع إلي "
                            "الترتيب الإفتراضي"
                        : "Clear all "
                            "Settings",
                    style:
                        TextStyle(fontSize: 15, fontWeight: FontWeight.bold)),
              ]),
            ),
            PopupMenuItem<String>(
              value: 'Export Excel',
              onTap: () async {
                if (_data == null) return;
                await ExcelService.generateExcel(savedList, _data!,
                    fileName: widget.report['nameAr']);
              },
              child: Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Text(
                    LocalConfigService.isArabic
                        ? "تصدير Excel"
                        : "Export Excel",
                    style: TextStyle(
                      fontSize: 15,
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                ],
              ),
            ),
          ],
        ),
        if (widget.isLite)
          Expanded(
            child: Row(
              children: [
                Spacer(),
                IconButton(
                  onPressed: () => Navigator.pop(context),
                  icon: Icon(LocalConfigService.isArabic
                      ? Icons.arrow_forward
                      : Icons.arrow_back),
                ),
              ],
            ),
          )
      ];

  Widget get childWidget => Column(
        children: [
          _getFilterTable(),
          if (isEditing)
            Center(
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  AppButtonIconText(
                    onPressed: () async {
                      savedData = await ReportDataService.saveTableColumnSort(
                          isEditing,
                          savedList,
                          context,
                          widget.report,
                          _sortColumnIndex ?? 0,
                          _sortAcd);
                      setState(() {
                        isEditing = !isEditing;
                      });
                    },
                    label: trans('Save Sort'),
                  ),
                  AppButtonIconText(
                    onPressed: () async {
                      setState(() {
                        isEditing = false;
                      });
                    },
                    label: trans('Cancel'),
                  ),
                ],
              ),
            ),
          SizedBox(height: 20),
          Expanded(child: _pageContent()),
        ],
      );

  Future<dynamic> _onPrint() async {
    try {
      var resultData = await EntityService.getData(
        EntityService.getEntity("Branch"),
      );
      // if (resultData.data == null) return;
      print('-------------start print');
      // var branchLogo =
      //     await ImageService.getImageBytes(resultData.data?[0]["logoId"]);

      print('-------------got branch');
      var doc = _getPrintablePdf(
          resultData.data?[0]["printTitleAr"] ?? '',
          resultData.data?[0]["printSubTitleAr"] ?? '',
          resultData.data?[0]["printFooterAr"] ?? '',
          resultData.data?[0]["printSubFooterAr"] ?? '',
          null);
      print('-------------pdf generated');
      var result = await Printing.layoutPdf(
        onLayout: (format) async => doc.save(),
        //name:widget.report['name${LocalConfigService.languagePrefex}'],
      );
      print('------------printed');
      if (kDebugMode) print(result);
    } catch (err, t) {
      print(err);
      print(t);
      AppNavigator.showMessage(
          context, 'خطأ اثناء طباعه التقرير', MessageType.error);
    }
  }

  Future<dynamic> _onSharePDF() async {
    try {
      var resultData = await EntityService.getData(
        EntityService.getEntity("Branch"),
      );
      // if (resultData.data == null) return;
      var branchLogo =
          await ImageService.getImageBytes(resultData.data?[0]["logoId"]);
      var doc = _getPrintablePdf(
        resultData.data?[0]["printTitleAr"] ?? '',
        resultData.data?[0]["printSubTitleAr"] ?? '',
        resultData.data?[0]["printFooterAr"] ?? '',
        resultData.data?[0]["printSubFooterAr"] ?? '',
        null,
      );
      var result = await Printing.sharePdf(
          bytes: await doc.save(),
          filename:
              '${widget.report['entity']?['name'][ConfigService.isRTL ? 'ar' : 'en'] ?? 'report'}.pdf');
      // var result = await Printing.layoutPdf(
      //   onLayout: (format) async => doc.save(),
      //   //name:widget.report['name${LocalConfigService.languagePrefex}'],
      // );
      if (kDebugMode) print(result);
    } catch (err, t) {
      print(err);
      print(t);
      AppNavigator.showMessage(
          context, 'خطأ اثناء تصدير التقرير', MessageType.error);
    }
  }

  pw.Document _getPrintablePdf(String titleAr, String subTitleAr,
      String footerAr, String subFooterAr, Uint8List? logoId) {
    final doc = pw.Document();
    DateTime printingDate = DateTime.now();
    String formattedDate =
        intl.DateFormat('yyyy-MM-dd   kk:mm').format(printingDate);
    doc.addPage(
      pw.MultiPage(
        maxPages: 500,
        orientation: _mode == "landscap"
            ? pw.PageOrientation.landscape
            : pw.PageOrientation.portrait,
        pageFormat: PdfPageFormat.a4,
        textDirection: pw.TextDirection.rtl,
        margin: pw.EdgeInsets.all(5),
        footer: (context) => pw.Column(children: [
          pw.Text(widget.report['footerAr'] ?? '',
              style: textStylePdf, textDirection: pw.TextDirection.rtl),
          pw.Row(mainAxisAlignment: pw.MainAxisAlignment.center, children: [
            pw.Row(children: [
              pw.Text(" ${formattedDate.toString()} ",
                  style: textStylePdf, textDirection: pw.TextDirection.ltr),
              pw.Text('تاريخ الطباعة : ',
                  style: textStylePdf, textDirection: pw.TextDirection.rtl),
            ]),
            pw.SizedBox(width: 40),
            pw.Row(children: [
              pw.Text(" ${LocalConfigService.user!.fullName.toString()} ",
                  style: textStylePdf, textDirection: pw.TextDirection.rtl),
              pw.Text('أسم المستخدم : ',
                  style: textStylePdf, textDirection: pw.TextDirection.rtl),
            ]),
          ]),
        ]),
        build: (c) => [
          if (titleAr.isNotEmpty &&
              subTitleAr.isNotEmpty &&
              footerAr.isNotEmpty &&
              subFooterAr.isNotEmpty)
            pw.Column(children: [
              pw.Row(children: [
                pw.Expanded(
                    child: pw.Row(children: [
                  logoId == null
                      ? pw.Container(width: 120, height: 120)
                      : pw.Container(
                          width: 120,
                          height: 120,
                          decoration: pw.BoxDecoration(
                              image: pw.DecorationImage(
                                  image: pw.MemoryImage(logoId),
                                  fit: pw.BoxFit.contain)),
                        ),
                  pw.Container(width: 40),
                ])),
                pw.SizedBox(width: 20),
                pw.Expanded(
                    child: pw.Column(
                  crossAxisAlignment: pw.CrossAxisAlignment.end,
                  children: [
                    pw.SizedBox(height: 5),
                    pw.Text(
                      titleAr.toString(),
                      style: textStyleBoldPdf,
                      // textDirection: pw.TextDirection.rtl
                    ),
                    pw.Text(subTitleAr.toString(),
                        style: textStylePdf,
                        textDirection: pw.TextDirection.rtl),
                    pw.Text(footerAr.toString(),
                        style: textStylePdf,
                        textDirection: pw.TextDirection.rtl),
                    pw.Text(subFooterAr.toString(),
                        style: textStylePdf,
                        textDirection: pw.TextDirection.rtl),
                  ],
                )),
              ]),
            ]),
          // pw.Center(
          //     child: pw.Text(
          //         widget.report['name${LocalConfigService.languagePrefix}'],
          //         style: textStylePdf.copyWith(fontSize: 15))),
          pw.SizedBox(height: 20),
          _getFilterTablePdf(),
          ...(_dataGrouped != null && _groupProperties != null
              ? _dataGrouped?.values
                      .map((e) => pw.Column(
                              mainAxisSize: pw.MainAxisSize.min,
                              children: [
                                pw.SizedBox(height: 20),
                                pw.Row(
                                  mainAxisAlignment: pw.MainAxisAlignment.start,
                                  children: _groupProperties!
                                      .map(
                                        (p) => pw.Padding(
                                          padding: const pw.EdgeInsets.only(
                                              left: 20.0, right: 20.0),
                                          child: pw.Row(
                                            crossAxisAlignment:
                                                pw.CrossAxisAlignment.start,
                                            mainAxisAlignment:
                                                pw.MainAxisAlignment.start,
                                            children: [
                                              _getCellReport(p, e.first),
                                              pw.SizedBox(width: 5),
                                              pw.Text(':'),
                                              pw.SizedBox(width: 5),
                                              pw.Text(p.nameAr,
                                                  style: textStylePdf,
                                                  maxLines: 1),
                                            ],
                                          ),
                                        ),
                                      )
                                      .toList(),
                                ),
                                ..._getDataTablePdf(
                                  e,
                                  e.length == 0
                                      ? null
                                      : _dataGroupedTotal![e.first[_groupBy]],
                                ),
                              ]))
                      .toList() ??
                  []
              : _getDataTablePdf(_data)),
        ],
      ),
    );

    return doc;
  }

  int _getWidthFlex(ReportProperty p) {
    switch (p.type) {
      case ValueTypes.Text:
        return 2;
      case ValueTypes.Boolean:
        return 1;
      case ValueTypes.Number:
        return 2;
      case ValueTypes.DateTime:
        return (p.format?.length ?? 0) > 8 ? 2 : 1;
      default:
        return 1;
    }
  }

  List<pw.Widget> _getDataTablePdf(List<Map<String, dynamic>>? data,
      [Map<String, dynamic>? total]) {
    var columns =
        (LocalConfigService.isArabic ? savedList.reversed : savedList);
    int i = 0;
    return [
      pw.Container(
          decoration: pw.BoxDecoration(
              color: PdfColors.grey200,
              border: pw.Border.all(color: PdfColors.black)),
          padding: pw.EdgeInsets.only(right: 10),
          child: pw.Row(
              children: columns
                  .where((element) =>
                      _groupProperties
                          ?.any((p) => p.dbName == element.dbName) !=
                      true)
                  .map<pw.Widget>(
                    (p) => pw.Expanded(
                        flex: _getWidthFlex(p),
                        child: pw.Align(
                          alignment: LocalConfigService.isArabic
                              ? pw.Alignment.centerRight
                              : pw.Alignment.centerLeft,
                          child: pw.Text(p.name, style: textStylePdf),
                        )),
                  )
                  .toList())),
      ...(data
              ?.map<pw.Widget>((d) => pw.Container(
                  decoration: i++ % 2 == 0
                      ? pw.BoxDecoration(
                          color: PdfColors.grey50,
                          border: pw.Border.all(color: PdfColors.black))
                      : pw.BoxDecoration(
                          border: pw.Border.all(color: PdfColors.black)),
                  padding: pw.EdgeInsets.only(right: 10),
                  child: pw.Row(
                    children: columns
                        .where((element) =>
                            _groupProperties
                                ?.any((p) => p.dbName == element.dbName) !=
                            true)
                        .map<pw.Widget>((p) => pw.Expanded(
                            flex: _getWidthFlex(p),
                            child: _getCellReport(p, d)))
                        .toList(),
                  )))
              .toList() ??
          []),
      pw.Container(
          decoration: pw.BoxDecoration(
              color: PdfColors.grey200,
              border: pw.Border.all(color: PdfColors.black)),
          child: pw.Row(
              //decoration: pw.BoxDecoration(color: PdfColors.grey200),
              children: columns
                  .map<pw.Widget>((p) => pw.Expanded(
                      flex: _getWidthFlex(p),
                      child: _getCellReport(
                          p,
                          total ??
                              (!showGrouped && _dataGrouped != null
                                  ? null
                                  : _totals),
                          totals: true)))
                  .toList()))
    ];
    // return pw.Table(
    //     //columnWidths: sortColumnIndex,
    //     border: pw.TableBorder(
    //         horizontalInside: pw.BorderSide(),
    //         top: pw.BorderSide(),
    //         bottom: pw.BorderSide(),
    //         right: pw.BorderSide(),
    //         left: pw.BorderSide()),
    //     children: (data
    //             ?.getRange(0, 47)
    //             ?.map<pw.TableRow>((d) => pw.TableRow(
    //                   children: savedList.reversed
    //                       .map<pw.Widget>((p) => _getCellReport(p, d))
    //                       .toList(),
    //                 ))
    //             .toList() ??
    //         [])
    //       ..insert(
    //         0,
    //         pw.TableRow(
    //           decoration: pw.BoxDecoration(color: PdfColors.grey300),
    //           children: savedList.reversed
    //               .map<pw.Widget>(
    //                 (p) => pw.Padding(
    //                   padding: pw.EdgeInsets.only(left: 5, right: 5),
    //                   child: pw.Align(
    //                     alignment: LocalConfigService.isArabic
    //                         ? pw.Alignment.centerRight
    //                         : pw.Alignment.centerLeft,
    //                     child: pw.Text(
    //                       p.name,
    //                       style: textStylePdf,
    //                     ),
    //                   ),
    //                 ),
    //               )
    //               .toList(),
    //         ),
    //       )
    //       ..add(pw.TableRow(
    //           decoration: pw.BoxDecoration(color: PdfColors.grey200),
    //           children: savedList.reversed
    //               .map<pw.Widget>(
    //                   (p) => _getCellReport(p, total ?? _totals, totals: true))
    //               .toList())));
  }

  pw.Widget _getCellReport(ReportProperty prop, data, {totals: false}) {
    var val = data == null ? null : data[prop.dbName];
    PdfColor color = PdfColors.black;
    if (prop.colorField?.isNotEmpty ?? false) {
      try {
        color = hexToColorPdf(data[prop.colorField]);
      } catch (err) {
        print(err);
      }
    } else if (prop.color?.isNotEmpty ?? false) {
      try {
        color = hexToColorPdf(prop.color!);
      } catch (err) {}
    }

    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return pw.Container();
    } else {
      if (val == null) return pw.Container();
    }
    var textStyle = totals
        ? textStyleBoldPdf
        : textStylePdf.copyWith(color: color, fontSize: 9);

    pw.Widget cell;
    switch (prop.type) {
      case ValueTypes.Text:
        cell = pw.Text(val, style: textStyle);
        break;
      case ValueTypes.Number:
        if (prop.format == '0')
          cell = pw.Text((val as num).toInt().toString(), style: textStyle);
        else
          cell = pw.Text((val as num).toDouble().toStringAsFixed(2),
              style: textStyle);
        break;
      case ValueTypes.DateTime:
        DateTime? date;
        if (val is num)
          date = DateTime.fromMillisecondsSinceEpoch(val.toInt()).toLocal();
        if (val is String) date = BaseModel.parseUTC(val)!;
        if (val is DateTime) date = val;

        cell = pw.Text(
            intl.DateFormat(prop.format ?? "yyyy-MM-dd HH:mm")
                .format(date!.toLocal()),
            style: textStyle.copyWith(fontSize: 7));
        break;
      default:
        cell = pw.Text('-');
        break;
    }
    if (prop.suffix != null ||
        prop.suffixField != null ||
        prop.prefix != null ||
        prop.prefixField != null) {
      List<pw.Widget> row = [];
      if (prop.prefix != null) row.add(pw.Text(prop.prefix!));
      if (prop.prefixField != null && data[prop.prefixField] != null)
        row.add(pw.Text(data[prop.prefixField!]));
      row.add(cell);
      if (prop.suffix != null) row.add(pw.Text(prop.suffix!));
      if (prop.suffixField != null && data[prop.suffixField] != null)
        row.add(pw.Text(data[prop.suffixField!]));

      return pw.Padding(
          padding: pw.EdgeInsets.only(left: 5, right: 5),
          child: pw.Row(
              mainAxisAlignment: pw.MainAxisAlignment.center, children: row));
    }
    return pw.Padding(
        padding: pw.EdgeInsets.only(left: 5, right: 5),
        child: pw.Align(
            alignment: LocalConfigService.isArabic
                ? pw.Alignment.centerRight
                : pw.Alignment.centerLeft,
            child: cell));
  }

  pw.Widget _getFilterTablePdf() {
    if (_filters == null) return pw.Container();
    return pw.Container(
        constraints: pw.BoxConstraints(maxWidth: 350),
        child: pw.Column(
          children: _filters!
              .map((e) => pw.Row(
                    children: [
                      pw.Expanded(
                          flex: 4,
                          child: pw.Text(e.getDisplayText(_filterData ?? {}),
                              style: textStylePdf)),
                      pw.Expanded(
                          flex: 3,
                          child: pw.Text(
                            e.nameAr,
                            style: textStylePdf,
                          )),
                    ],
                  ))
              .toList(),
        ));
  }

  List<DataColumn> getColumns() => savedList
      .map<DataColumn>((p) => DataColumn(
            numeric: p.type == ValueTypes.Number,
            onSort: (s, d) {
              _sortColumnIndex = s;
              if (savedData['sortColumnIndex'] == null)
                savedData['sortColumnIndex'] = {'sortColumnIndex': 0};

              savedData['sortColumnIndex']['sortColumnIndex'] = s;
              _sortAcd = d;
              if (savedData['sortAcd'] == null)
                savedData['sortAcd'] = {'sortAcd': true};
              savedData['sortAcd']['sortAcd'] = d;
              _onSort(p.dbName, p.type);
            },
            tooltip: p.nameEn,
            label: Container(
                padding: EdgeInsets.all(0),
                margin: EdgeInsets.all(0),
                child: isEditing
                    ? Column(
                        children: [
                          Row(
                            mainAxisAlignment: MainAxisAlignment.spaceAround,
                            children: [
                              //move right arrow
                              Tooltip(
                                message: LocalConfigService.isArabic
                                    ? "التحرك لليمين"
                                    : "Move Right",
                                child: InkWell(
                                    onTap: () {
                                      final index = savedList.indexOf(p);
                                      if (index > 0) {
                                        var temp;
                                        temp = savedList[index];
                                        savedList[index] = savedList[index - 1];
                                        savedList[index - 1] = temp;
                                        setState(() {});
                                      }
                                    },
                                    child: Icon(
                                      Icons.play_arrow,
                                      size: 16,
                                    )),
                              ),

                              //Delete Column Button
                              Tooltip(
                                message: LocalConfigService.isArabic
                                    ? "حذف"
                                    : "Delete",
                                child: InkWell(
                                    onTap: () {
                                      setState(() {
                                        savedList.remove(p);
                                      });
                                    },
                                    child: RotatedBox(
                                      quarterTurns: 2,
                                      child: Icon(
                                        Icons.hide_source,
                                        size: 12,
                                        color: Colors.red,
                                      ),
                                    )),
                              ),
                              //Move left arrow
                              Tooltip(
                                message: LocalConfigService.isArabic
                                    ? "التحرك لليسار"
                                    : "Move Left",
                                child: InkWell(
                                    onTap: () {
                                      final index = savedList.indexOf(p);
                                      if (index < savedList.length - 1) {
                                        var temp;
                                        temp = savedList[index];
                                        savedList[index] = savedList[index + 1];
                                        savedList[index + 1] = temp;
                                        setState(() {});
                                      }
                                    },
                                    child: RotatedBox(
                                        quarterTurns: 2,
                                        child: Icon(
                                          Icons.play_arrow,
                                          size: 16,
                                        ))),
                              ),
                            ],
                          ),
                          const SizedBox(height: 4),
                          Text(
                            p.name,
                            style: ThemeService.titleStyle
                                .copyWith(height: 0, fontSize: 14),
                          ),
                        ],
                      )
                    : Text(
                        p.name,
                        style: ThemeService.titleStyle
                            .copyWith(height: 0, fontSize: 14),
                      )),
          ))
      .toList();

  Widget _pageContent() {
    return _data == null
        ? Center(child: AppNavigator.getLoadingWidget(70))
        : showGrouped && _dataGrouped != null && _groupProperties != null
            ? ListView.builder(
                shrinkWrap: true,
                // physics: NeverScrollableScrollPhysics(),
                scrollDirection: Axis.vertical,
                itemCount: _dataGrouped?.values.length ?? 0,
                itemBuilder: (c, groupIndex) =>
                    Column(mainAxisSize: MainAxisSize.min, children: [
                      Row(
                        mainAxisAlignment: MainAxisAlignment.start,
                        children: _groupProperties!
                            .map(
                              (p) => Padding(
                                padding: const EdgeInsets.only(
                                    left: 20.0, right: 20.0),
                                child: Row(
                                  crossAxisAlignment: CrossAxisAlignment.start,
                                  mainAxisAlignment: MainAxisAlignment.start,
                                  children: [
                                    Text(p.nameAr, maxLines: 1),
                                    SizedBox(width: 5),
                                    Text(':'),
                                    SizedBox(width: 5),
                                    if ((_dataGrouped?.values
                                                .elementAt(groupIndex)
                                                .length ??
                                            0) >
                                        0)
                                      _getCellValue(
                                          p,
                                          _dataGrouped?.values
                                              .elementAt(groupIndex)
                                              .first),
                                  ],
                                ),
                              ),
                            )
                            .toList(),
                      ),
                      _getDataTable(
                          _dataGrouped?.values.elementAt(groupIndex),
                          _dataGrouped?.values.elementAt(groupIndex).length == 0
                              ? null
                              : _dataGroupedTotal![_dataGrouped?.values
                                  .elementAt(groupIndex)
                                  .first[_groupBy]])
                    ]))
            : _getDataTable(_data);
  }

  Widget _getDataTable(List<Map<String, dynamic>>? data,
      [Map<String, dynamic>? totals]) {
    return SfDataGrid(
        source: ReportDataSource(
            data: data,
            columns: savedList
                .where((element) =>
                    !showGrouped ||
                    _groupProperties?.any((p) => p.dbName == element.dbName) !=
                        true)
                .toList(),
            totals: totals),
        allowEditing: false,
        allowSorting: !showGrouped,
        allowFiltering: !showGrouped,
        allowColumnsResizing: false,
        allowMultiColumnSorting: !showGrouped,
        isScrollbarAlwaysShown: true,
        columnWidthMode: ColumnWidthMode.fitByColumnName,
        onQueryRowHeight: (details) {
          return details.getIntrinsicRowHeight(details.rowIndex);
        },
        onCellTap: (det) async {
          try {
            if (det.rowColumnIndex.rowIndex == 0) return;
            print(det.rowColumnIndex.rowIndex);
            print(det.column.columnName);
            if (data == null) return;
            var prop =
                savedList.firstWhere((e) => e.dbName == det.column.columnName);
            if (prop.drillDownEntity != null && prop.drillDownIdField != null) {
              var entity = prop.drillDownEntity!;
              if (prop.drillDownEntityDynamic)
                entity = data[det.rowColumnIndex.rowIndex + 1]
                    [prop.drillDownEntity!];
              if (entity.isEmpty ||
                  data[det.rowColumnIndex.rowIndex + 1][prop.drillDownIdField!]
                      .toString()
                      .isEmpty) return;
              AppNavigator.showLoading(context);
              var d = await EntityService.getOne(
                  entity,
                  data[det.rowColumnIndex.rowIndex + 1]
                          [prop.drillDownIdField!] ??
                      '');
              AppNavigator.pop(context);
              if (d == null) return;
              await AppNavigator.showAppDialogEx(
                  context,
                  () => EntityFormPage(
                        entity,
                        data: d,
                        isPopup: true,
                      ),
                  closeButton: false,
                  size: DialogBoxSizes.large);
              return;
            }
          } catch (err) {}
        },
        columns: savedList
            .where((element) =>
                !showGrouped ||
                _groupProperties?.any((p) => p.dbName == element.dbName) !=
                    true)
            .map<GridColumn>((p) => GridColumn(
                columnWidthMode: ColumnWidthMode.fitByColumnName,
                columnName: p.dbName,
                //this label is what will appear in the header of the column
                label: Container(
                  margin: EdgeInsets.all(4),
                  child: isEditing
                      ? Column(
                          children: [
                            Row(
                              mainAxisAlignment: MainAxisAlignment.spaceAround,
                              children: [
                                //move right arrow
                                Tooltip(
                                  message: LocalConfigService.isArabic
                                      ? "التحرك لليمين"
                                      : "Move Right",
                                  child: InkWell(
                                      onTap: () {
                                        final index = savedList.indexOf(p);
                                        if (index > 0) {
                                          var temp;
                                          temp = savedList[index];
                                          savedList[index] =
                                              savedList[index - 1];
                                          savedList[index - 1] = temp;
                                          setState(() {});
                                        }
                                      },
                                      child: Icon(
                                        Icons.play_arrow,
                                        size: 16,
                                      )),
                                ),

                                //Delete Column Button
                                Tooltip(
                                  message: LocalConfigService.isArabic
                                      ? "حذف"
                                      : "Delete",
                                  child: InkWell(
                                      onTap: () {
                                        setState(() {
                                          savedList.remove(p);
                                        });
                                      },
                                      child: RotatedBox(
                                        quarterTurns: 2,
                                        child: Icon(
                                          Icons.hide_source,
                                          size: 12,
                                          color: Colors.red,
                                        ),
                                      )),
                                ),
                                //Move left arrow
                                Tooltip(
                                  message: LocalConfigService.isArabic
                                      ? "التحرك لليسار"
                                      : "Move Left",
                                  child: InkWell(
                                      onTap: () {
                                        final index = savedList.indexOf(p);
                                        if (index < savedList.length - 1) {
                                          var temp;
                                          temp = savedList[index];
                                          savedList[index] =
                                              savedList[index + 1];
                                          savedList[index + 1] = temp;
                                          setState(() {});
                                        }
                                      },
                                      child: RotatedBox(
                                          quarterTurns: 2,
                                          child: Icon(
                                            Icons.play_arrow,
                                            size: 16,
                                          ))),
                                ),
                              ],
                            ),
                            const SizedBox(height: 4),
                            Text(
                              p.name,
                              style: ThemeService.titleStyle
                                  .copyWith(height: 0, fontSize: 14),
                              softWrap: true,
                              maxLines: 1,
                              overflow: TextOverflow.ellipsis,
                            ),
                          ],
                        )
                      : Text(
                          p.name,
                          style: ThemeService.titleStyle
                              .copyWith(height: 0, fontSize: 14),
                          softWrap: true,
                          maxLines: 1,
                          overflow: TextOverflow.ellipsis,
                        ),
                )))
            .toList());
  }

  Color hexToColor(String code) {
    if (code.startsWith('#'))
      return new Color(int.parse(code.substring(1, 7), radix: 16) + 0xFF000000);
    else
      return new Color(int.parse(code.substring(0, 6), radix: 16) + 0xFF000000);
  }

  PdfColor hexToColorPdf(String code) {
    return new PdfColor.fromHex(code);
  }

  DataCell _getCell(ReportProperty prop, data, {totals: false}) {
    var val = data[prop.dbName];
    dynamic onTap;

    if (prop.drillDownEntity != null && prop.drillDownIdField != null) {
      onTap = () async {
        AppNavigator.showLoading(context);
        var entity = prop.drillDownEntity!;
        if (prop.drillDownEntityDynamic) entity = data[prop.drillDownEntity!];
        var d =
            await EntityService.getOne(entity, data[prop.drillDownIdField!]);
        if (d == null) {
          AppNavigator.pop(context);
          return;
        }
        await AppNavigator.showAppDialogEx(
            context,
            () => EntityFormPage(
                  entity,
                  data: d,
                  isPopup: true,
                ),
            closeButton: false,
            size: DialogBoxSizes.large);
        AppNavigator.pop(context);
        return;
      };
    }

    Color color = Colors.black;
    if (prop.colorField?.isNotEmpty ?? false) {
      try {
        color = hexToColor(data[prop.colorField]);
      } catch (err) {}
    } else if (prop.color?.isNotEmpty ?? false) {
      try {
        color = hexToColor(prop.color!);
      } catch (err) {}
    }

    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return DataCell(Container());
    } else {
      if (val == null) return DataCell(Text('.'));
    }
    Widget? cell;
    switch (prop.type) {
      case ValueTypes.Text:
        cell = Text(val,
            style: totals
                ? ThemeService.titleStyle
                : TextStyle(fontSize: 12, color: color));
        break;
      case ValueTypes.Number:
        if (val is String) {
          val = num.parse(val);
        }
        if (prop.format == '0')
          cell = Text((val as num).toInt().toString(),
              style: totals
                  ? ThemeService.titleStyle
                  : TextStyle(fontSize: 12, color: color));
        else
          cell = Text((val as num).toDouble().toStringAsFixed(2),
              style: totals
                  ? ThemeService.titleStyle
                  : TextStyle(fontSize: 12, color: color));
        break;
      case ValueTypes.DateTime:
        DateTime? date = BaseModel.getDateEx(val);
        // if (val is num)
        //   date = DateTime.fromMillisecondsSinceEpoch(val.toInt()).toLocal();
        // if (val is String) date = BaseModel.parseUTC(val)!;
        // if (val is DateTime) date = val;

        cell = Text(
            date == null
                ? ''
                : intl.DateFormat(prop.format ?? "yyyy-MM-dd HH:mm")
                    .format(date.toLocal()),
            style: totals
                ? ThemeService.titleStyle
                : TextStyle(fontSize: 10, color: color));
        break;
      default:
        cell = Text('-');
        break;
    }

    if (prop.suffix != null ||
        prop.suffixField != null ||
        prop.prefix != null ||
        prop.prefixField != null) {
      List<Widget> row = [];
      if (prop.prefix != null) row.add(Text(prop.prefix!));
      if (prop.prefixField != null && data[prop.prefixField] != null)
        row.add(Text(data[prop.prefixField!]));
      row.add(SizedBox(width: 5));
      row.add(cell);
      row.add(SizedBox(width: 5));
      if (prop.suffix != null) row.add(Text(prop.suffix!));
      if (prop.suffixField != null && data[prop.suffixField] != null)
        row.add(Text(data[prop.suffixField!]));

      return DataCell(Row(children: row), onTap: onTap);
    }
    return DataCell(cell, onTap: onTap);
  }

  Text _getCellValue(ReportProperty prop, data, {totals: false}) {
    var val = data[prop.dbName];
    dynamic onTap;

    if (prop.drillDownEntity != null && prop.drillDownIdField != null) {
      onTap = () async {
        AppNavigator.showLoading(context);
        var entity = prop.drillDownEntity!;
        if (prop.drillDownEntityDynamic) entity = data[prop.drillDownEntity!];
        var d =
            await EntityService.getOne(entity, data[prop.drillDownIdField!]);
        if (d == null) {
          AppNavigator.pop(context);
          return;
        }
        await AppNavigator.showAppDialogEx(
            context,
            () => EntityFormPage(
                  entity,
                  data: d,
                  isPopup: true,
                ),
            closeButton: false,
            size: DialogBoxSizes.large);
        AppNavigator.pop(context);
        return;
      };
    }
    Color color = Colors.black;
    if (prop.colorField?.isNotEmpty ?? false) {
      try {
        color = hexToColor(data[prop.colorField]);
      } catch (err) {}
    }
    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0') return Text('');
    } else {
      if (val == null) return Text('.');
    }
    switch (prop.type) {
      case ValueTypes.Text:
        return Text(val,
            style: totals
                ? ThemeService.titleStyle
                : TextStyle(fontSize: 12, color: color));
      case ValueTypes.Number:
        if (val is String) {
          val = num.parse(val);
        }
        if (prop.format == '0')
          return Text((val as num).toInt().toString(),
              style: totals
                  ? ThemeService.titleStyle
                  : TextStyle(fontSize: 12, color: color));
        else
          return Text((val as num).toDouble().toStringAsFixed(2),
              style: totals
                  ? ThemeService.titleStyle
                  : TextStyle(fontSize: 12, color: color));
      case ValueTypes.DateTime:
        DateTime? date = BaseModel.getDateEx(val);
        // if (val is num)
        //   date = DateTime.fromMillisecondsSinceEpoch(val.toInt()).toLocal();
        // if (val is String) date = BaseModel.parseUTC(val)!;
        // if (val is DateTime) date = val;

        return Text(
            date == null
                ? ''
                : intl.DateFormat(prop.format ?? "yyyy-MM-dd HH:mm")
                    .format(date.toLocal()),
            style: totals
                ? ThemeService.titleStyle
                : TextStyle(fontSize: 10, color: color));
      default:
        return Text('-');
    }
  }

  Widget _getFilterTable() {
    if (_filters == null) return Container();
    return Container(
        constraints: BoxConstraints(maxWidth: 350),
        child: Column(
          children: _filters!.map((e) {
            var v = e.getDisplayText(_filterData ?? {});
            if (v.isEmpty || v == '-') v = 'غير محدد';
            return Row(
              children: [
                Expanded(flex: 3, child: Text(e.nameAr)),
                Expanded(
                  flex: 4,
                  child: Text(v),
                  // child: Text('sss'),
                ),
              ],
            );
          }).toList(),
        ));
  }

// _lowerFirst(String s) {
//   return '${s[0].toLowerCase()}${s.substring(1)}';
// }
}

class ReportDataSource extends DataGridSource {
  ReportDataSource(
      {this.data, required List<ReportProperty> columns, this.totals}) {
    _columns = Map<String, ReportProperty>();
    columns.forEach((e) {
      _columns[e.dbName] = e;
    });
    // if(data?.length > 0 )
    // data?.removeLast();
    _rows = data
            ?.map<DataGridRow>((e) => DataGridRow(
                    cells: columns.map(
                  (col) {
                    switch (col.type) {
                      case ValueTypes.Number:
                        return DataGridCell<num>(
                            columnName: col.dbName,
                            value: num.tryParse(
                                    MapService.getValue(e, col.dbName)
                                        .toString()) ??
                                0);
                      case ValueTypes.Text:
                        if (col.propertyType ==
                            PropertyTypes.TextMultiLanguage) {
                          var data = MapService.getValue(e, col.dbName);
                          return DataGridCell<String>(
                              columnName: col.dbName,
                              value: (data != null
                                      ? data[LocalConfigService.isArabic
                                          ? 'ar'
                                          : 'en']
                                      : data)
                                  .toString());
                        } else {
                          return DataGridCell<String>(
                              columnName: col.dbName,
                              value: MapService.getValue(e, col.dbName)
                                  .toString());
                        }

                      case ValueTypes.DateTime:
                        return DataGridCell<DateTime?>(
                            columnName: col.dbName,
                            value: BaseModel.getDateEx(
                                MapService.getValue(e, col.dbName)));
                      default:
                        return DataGridCell<String>(
                            columnName: col.dbName,
                            value:
                                MapService.getValue(e, col.dbName).toString());
                    }
                  },
                ).toList()))
            .toList() ??
        [];

    if (totals != null) {
      _rows.add(DataGridRow(
          cells: columns.map(
        (col) {
          switch (col.type) {
            case ValueTypes.Number:
              return DataGridCell<num>(
                  columnName: col.dbName, value: totals![col.dbName] as num);
            case ValueTypes.DateTime:
              return DataGridCell<DateTime?>(
                  columnName: col.dbName,
                  value: BaseModel.getDateEx(totals![col.dbName]));
            default:
              return DataGridCell<String>(
                  columnName: col.dbName,
                  value: totals![col.dbName]?.toString() ?? '');
          }
        },
      ).toList()));
    }
  }

  List<Map<String, dynamic>>? data;
  List<DataGridRow> _rows = [];
  late Map<String, ReportProperty> _columns;
  Map<String, dynamic>? totals;

  @override
  List<DataGridRow> get rows => _rows;

  @override
  DataGridRowAdapter? buildRow(DataGridRow row) {
    return DataGridRowAdapter(
        cells: row.getCells().map<Widget>((dataGridCell) {
      var prop = _columns[dataGridCell.columnName];
      return Container(
        alignment: Alignment.centerRight,
        padding: EdgeInsets.all(5.0),
        child: prop?.type == ValueTypes.DateTime && dataGridCell.value != null
            ? Text(intl.DateFormat(prop?.format ?? "yyyy-MM-dd HH:mm")
                .format((dataGridCell.value as DateTime).toLocal()))
            : Text(dataGridCell.value?.toString() ?? ''),
      );
    }).toList());
  }
}
